`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: PierceATronics
// Engineer: David Pierce Walker-Howell
// 
// Create Date:    11:25:11 04/18/2020 
// Design Name: 
// Module Name:    digilent_PMOD_AD1 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module digilent_PMOD_AD1 #(parameter SCLK_COUNTS=1, SCLK_COUNTER_LENGTH=2) (
				input SYS_CLK, RST, EN,
				input DATA_1_IN, DATA_2_IN,
				output reg RC, 
				output reg CS, SCLK,
				output reg [11:0] DATA_1_OUT, DATA_2_OUT);
	/*
		Input Parameters:
			SYS_CLK: The system clock(suggested 20MHz)
			RST: The reset, synchronous active high
			EN: Active high for enabling the ADC
			DATA_1_IN: Serial input data from channel 1 of AD1
			DATA_2_IN: Serial input data from channel 2 of AD1
		Output Parameters:
			RC: Receive complete flag 
			CS: The chip select (sync) to indicate start of a receive.
			SCLK: Serial clock to synchronous reading data from AD1.
			DATA_1_OUT: 12-bit output data from ADC
			DATA_2_OUT: 12-bit output data from ADC
	*/
	localparam IDLE=0,RECEIVE=1;
	reg [1:0] state;
	reg [8:0] shift_counter;
	reg [11:0] DATA_1_OUT_temp, DATA_2_OUT_temp;
	reg [SCLK_COUNTER_LENGTH-1:0] sclk_counter;
	reg rising_edge;
	
	//Divide the input clock by SCLK_COUNTS
	always @(posedge SYS_CLK) begin
	   if(RST) begin
	       sclk_counter <= 0;
	       SCLK <= 0;
	   end
	   else begin
	       if(sclk_counter == SCLK_COUNTS) begin
	           SCLK <= ~SCLK;
	           sclk_counter <= 0;
	       end
	       else begin
	           SCLK <= SCLK;
	           sclk_counter <= sclk_counter + 1;
	       end
	   end
	end
	
	always @(posedge SCLK or posedge RST) begin
	   if(RST) begin
	       state <= IDLE;
	       shift_counter <= 0;
	       RC <= 0;
	   end
	   else begin
	       if(1) begin//(sclk_counter == SCLK_COUNTS) & (SCLK == 1'b1)) begin
	           case(state) 
	               IDLE: begin
	                   CS <= 1;
	                   RC <= RC;
	                   if(shift_counter == 16) begin
	                       shift_counter <= 0;
	                       state <= RECEIVE;
	                   end
	                   else begin
	                       shift_counter <= shift_counter + 1;
	                       state <= IDLE;
	                   end
	               end
	               
	               RECEIVE: begin
	                   CS <= 0;
	                   
	                   if(shift_counter < 4) begin
	                       shift_counter <= shift_counter + 1;
	                       state <= RECEIVE;
	                       RC <= 0;
	                   end
	                   else if((shift_counter > 3) & (shift_counter < 16)) begin
	                       shift_counter <= shift_counter + 1;
	                       state <= RECEIVE;
	                       DATA_1_OUT_temp[15 - shift_counter] <= DATA_1_IN;
	                       DATA_2_OUT_temp[15 - shift_counter] <= DATA_2_IN;
	                       RC <= 0;
	                   end
	                   else if(shift_counter == 16) begin
	                       shift_counter <= 0;
	                       state <= IDLE;
	                       DATA_1_OUT <= DATA_1_OUT_temp;
	                       DATA_2_OUT <= DATA_2_OUT_temp;
	                       RC <= 1'b1;
	                   end
	               end
	           endcase
	       end
	       
	   end
	end
endmodule
