`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: PierceATronics
// Engineer: David Pierce Walker-Howell
// 
// Create Date: 03/27/2020 12:31:27 PM
// Design Name: 
// Module Name: simple_dual_port_mem
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

/*
This module is a generic simple dual port memeory that is read first the write.
Write is only available when the write enable is set high.
*/

module simple_dual_port_mem #(parameter ADDR_WIDTH=8, DATA_SIZE=16) (
    input CLK, RST, WRITE_EN,
    input signed [DATA_SIZE-1:0] data_in,
    input [ADDR_WIDTH-1:0] write_addr, read_addr,
    output reg signed[DATA_SIZE-1:0] data_out
    );
    
    //Declare the inferred memory
    reg signed [DATA_SIZE-1:0] mem [2**ADDR_WIDTH - 1:0];
    
	 /*
    //------------------FOR SIMULATION PURPOSE ONLY--------------
    initial begin
        mem[0] = 0;
        mem[1] = 0;
        mem[2] = 0;
        mem[3] = 0;
        mem[4] = 0;
    end
    //-----------------------------------------------------------
    */
	 
    always @(posedge CLK) begin
        if(WRITE_EN) 
            mem[write_addr] <= data_in;
        data_out <= mem[read_addr];  
    end
endmodule
