`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: PierceATronics
// Engineer: David Pierce Walker-Howell
// 
// Create Date:    09:21:30 03/31/2020 
// Design Name: 
// Module Name:    top_level 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module top_level(
	input CLK_27MHZ_FPGA,
	input HDR1_2, 				//UART RECIEVE RX PIN
	output DA2_SYNC,           //PMOD DA2 CS
	output DA2_DATA_1,           //PMOD DA2 DATA1
	output DA2_DATA_2,           //PMOD DA2 DATA2
	output DA2_SCLK,          //PMOD DA2 SCLK
	output AD1_CS,            //PMOD AD1 CHIP SELECT
	input AD1_DATA_1,         //PMOD AD1 serial data in 1
	input AD1_DATA_2,         //PMOD AD1 serial data in 2
	output AD1_SCLK,          //PMOD AD1 SCLK
	output GPIO_LED_0,
	output GPIO_LED_1
    );
	 
	 
	 //-----------------------UART RECEIVE MODULE------------------------------------------------
	 parameter RECEIVER_CLK_COUNTS = 88, RECEIVER_COUNTER_BIT_LENGTH = 7;
	 wire RECEIVER_CLK;
	 wire RXC;
	 wire [7:0] UDRRX;
	 //Instantiate the clock divider to divide the 27MHz clock into 16x 9600
	 clock_divider #(.COUNTS(RECEIVER_CLK_COUNTS), .COUNTER_BIT_LENGTH(RECEIVER_COUNTER_BIT_LENGTH)) receiver_clk 
	                (.CLK_IN(CLK_27MHZ_FPGA), .RST(1'b0), .CLK_OUT(RECEIVER_CLK));
	 
	 //Instantiate the receive RX module
	 receive rx (.CLK(RECEIVER_CLK), .RX_IN(HDR1_2), .RXE(1'b1), .RXC(RXC), .UDRRX(UDRRX));
	 
	 assign GPIO_LED_1 = (UDRRX == 8'h00) ? (1'b1) : (1'b0);
	 //---------------------------------------------------------------------------------------------
	 
	 //----------------------------------DATA UNPACK MODULE-----------------------------------------
	 //wire [7:0] filter_num;
	 wire [15:0] coeff_to_controller;
	 wire [2:0] coeff_addr_to_controller;
	 wire WRITE_EN;
	 
	 //Instantiate the unpack bytes module.
    unpack_bytes #(.NUM_FILTERS(1)) unpack (.SYS_CLK(CLK_27MHZ_FPGA), .RST(1'b0), .UDRRX(UDRRX), .RXC(RXC), 
                        .coeff_addr(coeff_addr_to_controller), 
                        .coeff_out(coeff_to_controller), .WRITE_EN(WRITE_EN));
	 //---------------------------------------------------------------------------------------------
	 
	 //-------------------------------COEFFICIENT CONTROLLER MODULE---------------------------------
	 wire [15:0] coeff_to_filter;
	 wire [2:0] coeff_addr_to_filter;
	 wire LOAD_OKAY;
	 wire LOAD;
	 
	 //Instantiate the coefficient controller that controls when to change the filter coefficients
    coefficient_controller coeff_controller(
                                  .SYS_CLK(CLK_27MHZ_FPGA), .RST(1'b0), .LOAD_EN(1'b1),
                                  .WRITE_EN(WRITE_EN), .write_addr(coeff_addr_to_controller), 
                                  .coeff_in(coeff_to_controller),
                                  .LOAD_OKAY(LOAD_OKAY), .LOAD(LOAD), .coeff_addr(coeff_addr_to_filter), 
                                  .coeff_out(coeff_to_filter));
	 //---------------------------------------------------------------------------------------------
	 
	 //---------------------------IIR FILTER MODULE-------------------------------------------------
	 parameter SAMPLE_CLK_COUNTS = 100, SAMPLE_CLK_COUNTER_BIT_LENGTH=9;
    wire SAMPLE_CLK;
    
    //Generate the sample clock. Typically 44100Hz
     clock_divider #(.COUNTS(SAMPLE_CLK_COUNTS), .COUNTER_BIT_LENGTH(SAMPLE_CLK_COUNTER_BIT_LENGTH)) 
            sample_clock (.CLK_IN(CLK_27MHZ_FPGA), .RST(1'b0), .CLK_OUT(SAMPLE_CLK));
	
	 //Assign the 8-bit input/output of AC97 codec to 16-bit format for the filter.
	 reg signed [15:0] x_in;
	 wire signed [15:0] y_out;
	 
    //Instantiate the IIR biquad filter
    //Instantiate the filter
    IIR_filter_biquad_df1 #(.BIT_WIDTH(16), .F(14)) filter_1
                                 (.SYS_CLK(CLK_27MHZ_FPGA), .SAMPLE_CLK(SAMPLE_CLK), .RST(1'b0), 
                                  .LOAD(LOAD), .coeff_addr(coeff_addr_to_filter), 
                                  .coeff(coeff_to_filter),
                                  .x_in(x_in), .y_out(y_out),
                                  .LOAD_OKAY(LOAD_OKAY));
    //--------------------------------------------------------------------//
	 //---------------------------------------------------------------------------------------------
	 
	 //-----------------------PMOD DAC2 OUTPUT----------------------------------------------------
	 wire WE;                         //Write enable
	 reg [11:0] to_DA2_data_1, to_DA2_data_2; //12-bit data to convert from digital to analog.
	 wire [1:0] MODE;
	 wire WC;
	 wire PMOD_CLK;
	 
	 assign WE = 1; //Set write enable 
	 assign MODE = 0; //Normal operation mode
	 
	 /*
	 clock_divider #(.COUNTS(1), .COUNTER_BIT_LENGTH(8)) 
	               pmod_clk_generation(.CLK_IN(CLK_27MHZ_FPGA), 
						.RST(1'b0), .CLK_OUT(PMOD_CLK));
	 */

	 //Instantiate module for PMOD DAC
    digilent_PMOD_DA2 da2_pmod(.SYS_CLK(CLK_27MHZ_FPGA), .RST(1'b0), .WE(WE), .DATA_1_IN(to_DA2_data_1), .DATA_2_IN(to_DA2_data_2),
                      .MODE(MODE), .WC(WC), .CS(DA2_SYNC), .SCLK(DA2_SCLK), .DATA_1_OUT(DA2_DATA_1), 
                      .DATA_2_OUT(DA2_DATA_2));
	 //-----------------------------------------------------------------------------------------
	 
	 
	//--------------------------PMOD AD1----------------------------------//
	wire [11:0] from_AD1_data_1, from_AD1_data_2;
	wire RC;
	digilent_PMOD_AD1 #(.SCLK_COUNTS(1), .SCLK_COUNTER_LENGTH(3)) 
							ad1(.SYS_CLK(CLK_27MHZ_FPGA), .RST(1'b0), .EN(1'b1),
								 .DATA_1_IN(AD1_DATA_1), .DATA_2_IN(AD1_DATA_2), 
								 .RC(RC), .CS(AD1_CS), .SCLK(AD1_SCLK),
								 .DATA_1_OUT(from_AD1_data_1), .DATA_2_OUT(from_AD1_data_2));

	reg signed [11:0] temp_1;
	reg signed [11:0] temp_2;
	reg signed [11:0] temp_3;
	reg signed [15:0] temp_4;
	
	always @(posedge CLK_27MHZ_FPGA) begin
		//DON'T CHANGE THIS!!! IT FINALLY WORKED
		//temp_1 <= from_AD1_data_1; //Convert unsigned data to signed
		//temp_2 <= temp_1 + 2048;   //Remove the DC bias of 1.5v => 0x800
		//temp_4[15:4] <= temp_2;    //place the 12 bit data into the 16 bit data
		x_in <= temp_4 >>> 2;      //Divide by 4 to ensure no overflow. All data is less than 1
		//x_in <= from_ac97_data >>> 2;
		//to_ac97_data <= y_out;
		temp_3 <= y_out[15:4] - 2048; //Add DC bias back in before writing to DAC
		to_DA2_data_1 <= temp_3[11:0]; //The filtered signal
		to_DA2_data_2 <= from_AD1_data_1; //Write the original input signal
	end
	
	//--------------------------------------------------------------------//
	 
	 
	 //-------------------BLINKING LED--------------------------------------------------------------
	 wire CLK_1HZ;
	 
	 //Instantiate the clock divider to divide the 27MHZ clock into 1Hz
	 clock_divider #(.COUNTS(13500000), .COUNTER_BIT_LENGTH(25)) clk_1hz 
						 (.CLK_IN(CLK_27MHZ_FPGA), .RST(1'b0), .CLK_OUT(CLK_1HZ));
	
	
	 assign GPIO_LED_0 = CLK_1HZ;
	 //----------------------------------------------------------------------------------------------
	 
endmodule
